/*! Tripetto rolling collector */
// Dependencies for this module:
//   ../../react
//   ../../tripetto-collector
//   ../../interfaces/style

declare module "tripetto-collector-standard-bootstrap" {
    export { Collector } from "tripetto-collector-standard-bootstrap/collector";
    export { ICollectorStyle } from "tripetto-collector-standard-bootstrap/interfaces/style";
    export { ICollectorOverrides } from "tripetto-collector-standard-bootstrap/interfaces/overrides";
    export { ICollectorSnapshot } from "tripetto-collector-standard-bootstrap/interfaces/snapshot";
    export { IBlockRenderer } from "tripetto-collector-standard-bootstrap/interfaces/renderer";
    export { IBlockHelper } from "tripetto-collector-standard-bootstrap/interfaces/block";
    export { usePauseDialog } from "tripetto-collector-standard-bootstrap/components/pause";
    export { run } from "tripetto-collector-standard-bootstrap/run";
}

declare module "tripetto-collector-standard-bootstrap/collector" {
    import * as React from "react";
    import { IDefinition, ISnapshot, Instance } from "tripetto-collector";
    import { ICollectorState } from "tripetto-collector-standard-bootstrap/interfaces/state";
    import { ICollectorStyle } from "tripetto-collector-standard-bootstrap/interfaces/style";
    import { ICollectorOverrides } from "tripetto-collector-standard-bootstrap/interfaces/overrides";
    import { ICollectorSnapshot } from "tripetto-collector-standard-bootstrap/interfaces/snapshot";
    import "bootstrap";
    import "./collector.scss";
    export class Collector extends React.PureComponent<{
        /** Specifies the definition to run. */
        readonly definition: IDefinition;
        /** Specifies the snapshot that should be restored. */
        readonly snapshot?: ISnapshot<ICollectorSnapshot>;
        /** Specifies the style settings for the collector. */
        readonly style?: ICollectorStyle;
        /** Specifies the overrides for the collector. */
        readonly overrides?: ICollectorOverrides;
        /** Specifies the view mode (`normal` by default). */
        readonly view?: "normal" | "preview" | "test";
        /** Specifies the window type for the collector (defaults to `full`). */
        readonly viewport?: "full" | "inline";
        /** Specifies the operation mode (`paginated` by default). */
        readonly mode?: "paginated" | "continuous" | "progressive";
        /** Specifies a function that is invoked when the collector is rendered. */
        readonly onReady?: (collector: Collector) => void;
        /** Invoked when there is a change. */
        readonly onChange?: (collector: Collector) => void;
        /** Specifies a function that is invoked when the collector is finished. */
        readonly onFinish?: (instance: Instance) => void;
        /**
         * Specifies a function that is invoked when the collector wants to
         * pause (return `true` in the supplied function when you want to wait with
         * the actual pause until the `pause` function is invoked)
         */
        readonly onPause?: (collector: Collector) => boolean | void;
        /** Specifies a function that is invoked when the collector is paused. */
        readonly onPaused?: (snapshot: ISnapshot<ICollectorSnapshot>) => void;
        /** Specifies a function that is invoked when the edit button of a block in preview mode is clicked. */
        readonly onEditRequest?: (nodeId: string) => void;
        /** Specifies attachment related settings. */
        readonly onAttachment?: {
            /** Specifies the function to retrieve an attachment. */
            get: (file: string) => Promise<Blob>;
            /** Specifies the function that is invoked when a file is attached. */
            put: (file: File, onProgress: (percent?: number) => void) => Promise<string>;
            /** Specifies the function that is invoked when an attachment is deleted. */
            delete: (file: string) => Promise<void>;
        };
    }> {
        readonly element: React.RefObject<HTMLElement>;
        readonly state: ICollectorState;
        /** Retrieves the definition. */
        readonly definition: IDefinition | undefined;
        /** Retrieves the name of the definition. */
        readonly name: string;
        /** Retrieves if the collector is empty. */
        readonly isEmpty: boolean;
        /** Retrieves if the collector is running. */
        readonly isRunning: boolean;
        /** Retrieves if the collector is stopped. */
        readonly isStopped: boolean;
        /** Retrieves if the collector has finished (without pausing or stopping it). */
        readonly isFinished: boolean;
        /** Retrieves if the collector is reloading. */
        readonly isReloading: boolean;
        /** Sets the current style. */
        style: ICollectorStyle;
        /** Sets the current overrides. */
        overrides: ICollectorOverrides;
        /** Sets the current view mode. */
        view: "normal" | "preview" | "test";
        /** Sets the current operation mode. */
        mode: "paginated" | "continuous" | "progressive";
        render(): React.ReactNode;
        componentDidMount(): void;
        componentDidUpdate(): void;
        componentWillUnmount(): void;
        /** Start the collector. */
        start(): void;
        /** Resume a collector that was previously paused. */
        resume(snapshot: ISnapshot<ICollectorSnapshot>): boolean;
        /** Pauses the collector. */
        pause(): ISnapshot<ICollectorSnapshot> | undefined;
        /** Requests a pause. */
        requestPause(): void;
        /** Stop the collector. */
        stop(): void;
        /** Restarts the collector. */
        restart(): void;
        /** Reload with a new definition. */
        reload(definition: IDefinition): void;
        /** Call this function when the parent element is resized. */
        resize(): void;
        /** Shows the pause dialog. */
        showPauseDialog(whenDone: (emailAddress: string, collector: this) => void): void;
        /** Request a preview of the specified node. */
        requestPreview(nodeId: string): void;
    }
}

declare module "tripetto-collector-standard-bootstrap/interfaces/style" {
    export type TFonts =
        | ""
        | "Arial, Helvetica, Sans-Serif"
        | "Arial Black, Gadget, Sans-Serif"
        | "Comic Sans MS, Textile, Cursive, Sans-Serif"
        | "Courier New, Courier, Monospace"
        | "Georgia, Times New Roman, Times, Serif"
        | "Impact, Charcoal, Sans-Serif"
        | "Lucida Console, Monaco, Monospace"
        | "Lucida Sans Unicode, Lucida Grande, Sans-Serif"
        | "Palatino Linotype, Book Antiqua, Palatino, Serif"
        | "Tahoma, Geneva, Sans-Serif"
        | "Times New Roman, Times, Serif"
        | "Trebuchet MS, Helvetica, Sans-Serif"
        | "Verdana, Geneva, Sans-Serif";
    export type TBootstrapStyles = "primary" | "secondary" | "success" | "info" | "warning" | "danger" | "light" | "dark";
    export type TBootstrapStylesOutline =
        | "outline-primary"
        | "outline-secondary"
        | "outline-success"
        | "outline-info"
        | "outline-warning"
        | "outline-danger"
        | "outline-light"
        | "outline-dark";
    export interface ICollectorStyle {
        readonly backgroundColor?: string;
        readonly textFont?: TFonts;
        readonly textColor?: string;
        readonly mode?: "paginated" | "continuous" | "progressive";
        readonly showEnumerators?: boolean;
        readonly showPageIndicators?: boolean;
        readonly showProgressbar?: boolean;
        readonly backgroundImage?: {
            readonly url: string;
            readonly size: "auto" | "100% auto" | "auto 100%" | "cover" | "contain" | "repeat";
        };
        readonly form?: {
            readonly inputStyle: TBootstrapStyles;
            readonly unselectedStyle: TBootstrapStyles | TBootstrapStylesOutline;
            readonly selectedStyle: TBootstrapStyles | TBootstrapStylesOutline;
            readonly neutralStyle: TBootstrapStyles | TBootstrapStylesOutline;
            readonly positiveStyle: TBootstrapStyles | TBootstrapStylesOutline;
            readonly negativeStyle: TBootstrapStyles | TBootstrapStylesOutline;
        };
        readonly buttons?: {
            readonly nextStyle: TBootstrapStyles | TBootstrapStylesOutline;
            readonly nextLabel?: string;
            readonly backStyle: TBootstrapStyles | TBootstrapStylesOutline;
            readonly backLabel?: string;
            readonly completeStyle: TBootstrapStyles | TBootstrapStylesOutline;
            readonly completeLabel?: string;
            readonly pauseStyle: TBootstrapStyles | TBootstrapStylesOutline;
        };
        readonly navigation?: {
            readonly progressbarStyle: TBootstrapStyles;
            readonly paginationStyle: TBootstrapStyles;
        };
    }
}

declare module "tripetto-collector-standard-bootstrap/interfaces/overrides" {
    export interface ICollectorOverrides {
        readonly removeBranding?: boolean;
        readonly confirmationTitle?: string;
        readonly confirmationSubtitle?: string;
        readonly confirmationText?: string;
        readonly confirmationImage?: string;
        readonly confirmationButton?:
            | {
                  readonly label: string;
                  readonly url: string;
              }
            | "off";
    }
}

declare module "tripetto-collector-standard-bootstrap/interfaces/snapshot" {
    export interface ICollectorSnapshot {
        readonly b?: {
            [key: string]: false | true | undefined;
        };
    }
}

declare module "tripetto-collector-standard-bootstrap/interfaces/renderer" {
    import { NodeBlock } from "tripetto-collector";
    import { IBlockHelper } from "tripetto-collector-standard-bootstrap/interfaces/block";
    export interface IBlockRenderer extends NodeBlock {
        render: (block: IBlockHelper) => React.ReactNode;
    }
}

declare module "tripetto-collector-standard-bootstrap/interfaces/block" {
    import * as React from "react";
    import { ICollectorStyle } from "interfaces/style";
    export interface IBlockHelper {
        /** Contains the style. */
        style: ICollectorStyle;
        /** Parsed markdown name. */
        name: (required?: boolean, labelFor?: string) => React.ReactNode;
        /** Parsed markdown description. */
        description: React.ReactNode;
        /** Parsed image. */
        imageFromURL: (imageURL?: string, imageWidth?: string) => React.ReactNode;
        /** Parsed markdown explanation. */
        explanation: (label?: string) => React.ReactNode;
        /** Parsed markdown placeholder. */
        placeholder: string;
        /** Parsed markdown placeholder or name (single line). */
        placeholderOrName: (required?: boolean) => React.ReactNode;
        /** Tab-index for the block. */
        tabIndex: number;
        /** Specifies if the block validation failed. */
        isFailed: boolean;
        /** Updates a block. */
        update: () => void;
        /** The block gets focus. */
        onFocus: () => void;
        /** The block loses focus. */
        onBlur: () => void;
        /** Specifies the function that is invoked when a file is attached. */
        onAttachment?: {
            get: (file: string) => Promise<Blob>;
            put: (file: File, onProgress: (percent?: number) => void) => Promise<string>;
            delete: (file: string) => Promise<void>;
        };
    }
}

declare module "tripetto-collector-standard-bootstrap/components/pause" {
    import { Collector } from "tripetto-collector-standard-bootstrap/collector";
    import { ISnapshot } from "tripetto-collector";
    import { ICollectorStyle } from "tripetto-collector-standard-bootstrap/interfaces/style";
    import { ICollectorSnapshot } from "tripetto-collector-standard-bootstrap/interfaces/snapshot";
    import * as React from "react";
    export const pauseDialog: (
        ref: React.RefObject<HTMLDivElement>,
        style: ICollectorStyle,
        whenPauseDialogDone: (emailAddress: string) => void
    ) => JSX.Element;
    export const usePauseDialog: (
        saveSnapshot: (emailAddress: string, snapshot: ISnapshot<ICollectorSnapshot>) => void
    ) => (collector: Collector) => boolean;
}

declare module "tripetto-collector-standard-bootstrap/run" {
    import { IDefinition, ISnapshot, Instance } from "tripetto-collector";
    import { Collector } from "tripetto-collector-standard-bootstrap/collector";
    import { ICollectorStyle } from "tripetto-collector-standard-bootstrap/interfaces/style";
    import { ICollectorOverrides } from "tripetto-collector-standard-bootstrap/interfaces/overrides";
    import { ICollectorSnapshot } from "tripetto-collector-standard-bootstrap/interfaces/snapshot";
    import * as React from "react";
    export function run(props: {
        /** Specifies the parent element for the collector. */
        readonly element: HTMLElement | null;
        /** Specifies the definition to run. */
        readonly definition?: IDefinition | Promise<IDefinition>;
        /** Specifies the snapshot that should be restored. */
        readonly snapshot?: ISnapshot<ICollectorSnapshot> | Promise<ISnapshot<ICollectorSnapshot> | undefined>;
        /** Specifies the styles for the collector. */
        readonly style?: ICollectorStyle | Promise<ICollectorStyle | undefined>;
        /** Specifies the overrides for the collector. */
        readonly overrides?: ICollectorOverrides | Promise<ICollectorOverrides>;
        /** Specifies the view mode (`normal` by default). */
        readonly view?: "normal" | "preview" | "test";
        /** Specifies the window type for the collector (defaults to `full`). */
        readonly viewport?: "full" | "inline";
        /** Specifies the operation mode (`paginated` by default). */
        readonly mode?: "paginated" | "continuous" | "progressive";
        /** Specifies if the collector needs to be automatically resized when the host window is resized. */
        readonly autoResize?: boolean;
        /** Specifies a React ref for the collector component. */
        readonly ref?: React.RefObject<Collector>;
        /** Specifies a function that is invoked when the collector is rendered. */
        readonly onReady?: (collector: Collector) => void;
        /** Invoked when there is a change. */
        readonly onChange?: (collector: Collector) => void;
        /** Specifies a function that is invoked when the collector is finished. */
        readonly onFinish?: (instance: Instance) => void;
        /**
         * Specifies a function that is invoked when the collector wants to
         * pause (return `true` in the supplied function when you want to wait with
         * the actual pause until the `pause` function is invoked)
         */
        readonly onPause?: (collector: Collector) => boolean | void;
        /** Specifies a function that is invoked when the collector is paused. */
        readonly onPaused?: (snapshot: ISnapshot<ICollectorSnapshot>) => void;
        /** Specifies a function that is invoked when the edit button of a block in preview mode is clicked. */
        readonly onEditRequest?: (nodeId: string) => void;
        /** Specifies attachment related settings. */
        readonly onAttachment?: {
            /** Specifies the function to retrieve an attachment. */
            get: (file: string) => Promise<Blob>;
            /** Specifies the function that is invoked when a file is attached. */
            put: (file: File, onProgress: (percent?: number) => void) => Promise<string>;
            /** Specifies the function that is invoked when an attachment is deleted. */
            delete: (file: string) => Promise<void>;
        };
    }): Promise<{
        /** A reference to the collector component. */
        readonly collector: Collector;
        /** Specifies the definition. */
        definition: IDefinition;
        /** Specifies the style. */
        style: ICollectorStyle | undefined;
        /** Specifies the overrides. */
        overrides: ICollectorOverrides | undefined;
        /** Specifies the view mode. */
        view: "normal" | "preview" | "test";
        /** Specifies the operation mode. */
        mode: "paginated" | "continuous" | "progressive";
        /** Retrieves the height of the content in pixels. */
        readonly height: number;
        /** Request a preview of specified node. */
        readonly requestPreview: (nodeId: string) => void;
        /** Requests a pause. */
        readonly requestPause: () => void;
        /** Pauses the collector. */
        readonly pause: () => void;
        /** Indicates the parent element is resized. */
        readonly resize: () => void;
        /** Destroys the component. */
        readonly destroy: () => void;
    }>;
}

declare module "tripetto-collector-standard-bootstrap/interfaces/state" {
    import { ICollectorStyle } from "tripetto-collector-standard-bootstrap/interfaces/style";
    import { ICollectorOverrides } from "tripetto-collector-standard-bootstrap/interfaces/overrides";
    export interface ICollectorState {
        style: ICollectorStyle;
        overrides: ICollectorOverrides;
        view: "normal" | "preview" | "test";
        mode: "paginated" | "continuous" | "progressive";
        changes: number;
        focus: {
            [key: string]: boolean | undefined;
        };
        activate?: string;
    }
}
